/**
 * This code is based on:
 * https://github.com/conorhastings/react-native-syntax-highlighter
 */
import React, { useMemo } from "react";
import { Text, Platform, View } from "react-native";
import { Prism } from "react-syntax-highlighter";
import SyntaxHighlighter from "react-syntax-highlighter";
import * as prismStyles from "react-syntax-highlighter/dist/esm/styles/prism";
import * as hljsStyles from "react-syntax-highlighter/dist/esm/styles/hljs";
import { ScrollView } from "react-native-gesture-handler";
import { createStyleObject } from "./utils";
import styles from "./styes";
const styleCache = new Map();
const topLevelPropertiesToRemove = [
    "color",
    "textShadow",
    "textAlign",
    "whiteSpace",
    "wordSpacing",
    "wordBreak",
    "wordWrap",
    "lineHeight",
    "MozTabSize",
    "OTabSize",
    "tabSize",
    "WebkitHyphens",
    "MozHyphens",
    "msHyphens",
    "hyphens",
    "fontFamily",
];
/**
 * TODO: improve types
 */
export function generateNewStylesheet({ stylesheet, highlighter, }) {
    if (styleCache.has(stylesheet)) {
        return styleCache.get(stylesheet);
    }
    stylesheet = Array.isArray(stylesheet) ? stylesheet[0] : stylesheet;
    const transformedStyle = Object.entries(stylesheet).reduce((newStylesheet, [className, style]) => {
        newStylesheet[className] = Object.entries(style).reduce((newStyle, [key, value]) => {
            if (key === "overflowX" || key === "overflow") {
                newStyle.overflow = value === "auto" ? "scroll" : value;
            }
            else if (value.includes("em")) {
                const [num] = value.split("em");
                newStyle[key] = Number(num) * 16;
            }
            else if (key === "background") {
                newStyle.backgroundColor = value;
            }
            else if (key === "display") {
                return newStyle;
            }
            else {
                newStyle[key] = value;
            }
            return newStyle;
        }, {});
        return newStylesheet;
    }, {});
    const topLevel = highlighter === "prism"
        ? transformedStyle['pre[class*="language-"]']
        : transformedStyle.hljs;
    const defaultColor = (topLevel && topLevel.color) || "#000";
    topLevelPropertiesToRemove.forEach((property) => {
        if (topLevel[property]) {
            delete topLevel[property];
        }
    });
    if (topLevel.backgroundColor === "none") {
        delete topLevel.backgroundColor;
    }
    const codeLevel = transformedStyle['code[class*="language-"]'];
    if (highlighter === "prism" && !!codeLevel) {
        topLevelPropertiesToRemove.forEach((property) => {
            if (codeLevel[property]) {
                delete codeLevel[property];
            }
        });
        if (codeLevel.backgroundColor === "none") {
            delete codeLevel.backgroundColor;
        }
    }
    styleCache.set(stylesheet, { transformedStyle, defaultColor });
    return { transformedStyle, defaultColor };
}
function createChildren({ stylesheet, fontSize, fontFamily, }) {
    let childrenCount = 0;
    return (children, defaultColor) => {
        childrenCount += 1;
        return children.map((child, i) => createNativeElement({
            node: child,
            stylesheet,
            key: `code-segment-${childrenCount}-${i}`,
            defaultColor,
            fontSize,
            fontFamily,
        }));
    };
}
function createNativeElement({ node, stylesheet, key, defaultColor, fontFamily, fontSize = 12, }) {
    const { properties, type, tagName: TagName, value } = node;
    const startingStyle = { fontFamily, fontSize, height: fontSize + 5 };
    if (type === "text") {
        return (React.createElement(Text, { key: key, style: Object.assign({ color: defaultColor }, startingStyle) }, value));
    }
    else if (TagName) {
        const childrenCreator = createChildren({
            stylesheet,
            fontSize,
            fontFamily,
        });
        const style = createStyleObject(properties.className, Object.assign({ color: defaultColor }, properties.style, startingStyle), stylesheet);
        const children = childrenCreator(node.children, style.color || defaultColor);
        return (React.createElement(Text, { key: key, style: style }, children));
    }
    else {
        return null;
    }
}
function nativeRenderer({ defaultColor, fontFamily, fontSize, dark, startingLineNumber, }) {
    return ({ rows, stylesheet }) => {
        const width = 16 + (1 + Math.floor(Math.log10(rows.length))) * fontSize * 0.75;
        return rows.map((node, i) => {
            return (React.createElement(View, { key: `row-wrapper${i + startingLineNumber}`, style: styles.row_container },
                React.createElement(View, { style: [
                        styles.number_container,
                        {
                            width: width,
                            backgroundColor: dark ? "#000" : "#fff",
                        },
                    ] },
                    React.createElement(Text, { style: [
                            styles.number,
                            {
                                color: dark ? "#fff" : "#000",
                                fontSize: fontSize,
                            },
                        ] }, `${i + startingLineNumber}`)),
                React.createElement(View, { style: [
                        styles.text_container,
                        {
                            paddingLeft: width + 4,
                        },
                    ] },
                    React.createElement(Text, null, createNativeElement({
                        node,
                        stylesheet,
                        key: `code-segment-${i}`,
                        defaultColor,
                        fontFamily,
                        fontSize,
                    })))));
        });
    };
}
export const PreTag = (props) => {
    return (React.createElement(ScrollView, Object.assign({}, props, { horizontal: true, contentContainerStyle: styles.pretag })));
};
export const CodeTag = (props) => {
    return (React.createElement(ScrollView, Object.assign({}, props, { horizontal: false, contentContainerStyle: styles.codetag })));
};
const fontFamilyDef = Platform.OS == "ios" ? "Menlo-Regular" : "monospace";
const fontSizeDef = 12;
const prismDefaultStyle = prismStyles.atomDark;
const defaultStyle = hljsStyles.agate;
const RNSyntaxHighlighter = ({ fontFamily = fontFamilyDef, fontSize = fontSizeDef, children, dark = true, highlighter = "highlighterjs", style = highlighter === "prism" ? prismDefaultStyle : defaultStyle, startingLineNumber = 1, ...rest }) => {
    const { transformedStyle, defaultColor } = useMemo(() => {
        return generateNewStylesheet({
            stylesheet: style,
            highlighter,
        });
    }, [style, highlighter]);
    const Highlighter = useMemo(() => (highlighter === "prism" ? Prism : SyntaxHighlighter), [highlighter]);
    const renderer = useMemo(() => {
        return nativeRenderer({
            defaultColor,
            fontFamily,
            fontSize,
            dark,
            startingLineNumber,
        });
    }, [defaultColor, fontFamily, fontSize, dark, startingLineNumber]);
    return (React.createElement(Highlighter, Object.assign({}, rest, { PreTag: PreTag, CodeTag: CodeTag, style: transformedStyle, showLineNumbers: false, showInlineLineNumbers: false, wrapLines: true, horizontal: true, renderer: renderer }), children));
};
export default React.memo(RNSyntaxHighlighter);
